/*
 * Copyright (c) 2009 Mysema Ltd.
 * All rights reserved.
 * 
 */
package com.mysema.commons.jetty;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.apache.commons.io.IOUtils;
import org.mortbay.jetty.Connector;
import org.mortbay.jetty.Handler;
import org.mortbay.jetty.Server;
import org.mortbay.jetty.handler.DefaultHandler;
import org.mortbay.jetty.nio.SelectChannelConnector;
import org.mortbay.jetty.security.SslSocketConnector;
import org.mortbay.jetty.webapp.WebAppContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * JettyHelper provides convenience methods for starting Jetty instances with compact configuration options
 * 
 * @author Timo Westkamper
 * @author Marko Lavikainen
 * @version $Id$
 */
public class JettyHelper {

    private static final Logger logger = LoggerFactory.getLogger(JettyHelper.class);

    private static final Map<Integer, Server> servers = new HashMap<Integer, Server>();

    private static final File keystoreLocation;
    
    static{
        try {
            keystoreLocation = File.createTempFile("keystore", null);
            InputStream in = JettyHelper.class.getResourceAsStream("/keystore");
            OutputStream out = new FileOutputStream(keystoreLocation);
            try{
                IOUtils.copy(in, out);    
            }finally{
                out.close();
                in.close();                
            }      
        } catch (IOException e) {
           throw new RuntimeException(e.getMessage(), e);
        }
        
    }
    
    public static boolean runningAt(int port) {
        return servers.containsKey(port);
    }

    public static WebAppContext startJetty(String warPath, String contextPath, int port,
            int securePort) throws Exception {
        WebAppContext webappcontext = new WebAppContext();
        webappcontext.setContextPath(contextPath);
        webappcontext.setWar(warPath);
        if (new File(warPath).isFile()) {
            webappcontext.setExtractWAR(true);
        }
        startJetty(port, securePort, webappcontext);
        return webappcontext;
    }

    public static void startJetty(int port, int securePort,
            WebAppContext... contexts) throws Exception {
        if (servers.containsKey(port)) {
            logger.error("Already one Jetty instance running on port " + port, new RuntimeException());
            return;
        }

        if (contexts == null || contexts.length == 0) {
            throw new IllegalArgumentException();
        }

        final Server server = new Server();
        servers.put(port, server);

        List<Connector> connectors = new ArrayList<Connector>(2);
        
        // http
        if (port > 0){
            SelectChannelConnector httpConnector = new SelectChannelConnector();
            httpConnector.setPort(port);
            httpConnector.setConfidentialPort(securePort);    
            connectors.add(httpConnector);
        }        

        // https
        if (securePort > 0){
            SslSocketConnector httpsConnector = new SslSocketConnector();
            httpsConnector.setPort(securePort);
            httpsConnector.setMaxIdleTime(60000);
            httpsConnector.setKeystore(keystoreLocation.getAbsolutePath());
            httpsConnector.setPassword("OBF:1vny1zlo1x8e1vnw1vn61x8g1zlu1vn4");
            httpsConnector.setKeyPassword("OBF:1u2u1wml1z7s1z7a1wnl1u2g");
            connectors.add(httpsConnector);
        }
        
        server.setConnectors(connectors.toArray(new Connector[connectors.size()]));

        List<Handler> handlers = new LinkedList<Handler>();
        for (WebAppContext context : contexts) {
            handlers.add(context);
        }
        handlers.add(new DefaultHandler());
        server.setHandlers(handlers.toArray(new Handler[0]));
        
        server.start();
        logger.info("Jetty running on port " + port);
    }

    public static void stopJettyAtPort(int port) throws Exception {
        Server server = servers.remove(port);
        if (server != null) {
            server.stop();
            logger.info("Stopped Jetty on port " + port);
        } else {
            logger.warn("No Jetty instance running on port " + port, new RuntimeException());
        }
    }
    
}
